@@ -76,41 +76,17 @@ class ckbytelist(PyKCS11.LowLevel.ckbytelist):
     add a __repr__() method to the LowLevel equivalent
     """
 
-    def __init__(self, data=[]):
-        # default size of the vector
-        size = 0
-        super(ckbytelist, self).__init__(size)
-
-        # No value to initialize
+    def __init__(self, data=None):
         if data is None:
-            return
-
-        # b'abc'
-        if isinstance(data, bytes):
-            self.reserve(len(data))
-            for x in data:
-                if sys.version_info[0] <= 2:
-                    # Python 2
-                    v = ord(x)
-                else:
-                    # Python 3 and more
-                    v = x
-                self.append(v)
-
-        # "abc"
+            data = 0
         elif isinstance(data, str):
-            tmp = bytes(data, "utf-8")
-            self.reserve(len(tmp))
-            for x in tmp:
-                self.append(x)
-
-        # [141, 142, 143]
-        elif isinstance(data, list) or isinstance(data, ckbytelist):
-            self.reserve(len(data))
-            for c in range(len(data)):
-                self.append(data[c])
+            data = data.encode("utf-8")
+        elif isinstance(data, (bytes, list, ckbytelist)):
+            data = bytes(data)
         else:
             raise PyKCS11.PyKCS11Error(-3, text=str(type(data)))
+        super(ckbytelist, self).__init__(data)
+
 
     def __repr__(self):
         """
@@ -470,25 +446,8 @@ class PyKCS11Lib(object):
                 PyKCS11.LowLevel._LowLevel and \
                 PyKCS11.LowLevel._LowLevel.__name__:
 
-            # in case NO library was found and used
-            if not hasattr(self, "pkcs11dll_filename"):
-                return
-
-            # in case the load failed
-            if self.pkcs11dll_filename not in PyKCS11Lib._loaded_libs:
-                return
-
-            # decrease user number
-            PyKCS11Lib._loaded_libs[self.pkcs11dll_filename]["nb_users"] -= 1
-
-            if PyKCS11Lib._loaded_libs[self.pkcs11dll_filename]["nb_users"] == 0:
-                # unload only if no more used
-                self.lib.Unload()
-
-            # remove unused entry
-            # the case < 0 happens if lib loading failed
-            if PyKCS11Lib._loaded_libs[self.pkcs11dll_filename]["nb_users"] <= 0:
-                del PyKCS11Lib._loaded_libs[self.pkcs11dll_filename]
+            # unload the library
+            self.unload()
 
     def load(self, pkcs11dll_filename=None, *init_string):
         """
@@ -506,8 +465,11 @@ class PyKCS11Lib(object):
             if pkcs11dll_filename is None:
                 raise PyKCS11Error(-1, "No PKCS11 library specified (set PYKCS11LIB env variable)")
 
-        # remember the lib file name
-        self.pkcs11dll_filename = pkcs11dll_filename
+        if hasattr(self, "pkcs11dll_filename"):
+            self.unload() # unload the previous library
+            # if the instance was previously initialized,
+            # create a new low level library object for it
+            self.lib = PyKCS11.LowLevel.CPKCS11Lib()
 
         # if the lib is already in use: reuse it
         if pkcs11dll_filename in PyKCS11Lib._loaded_libs:
@@ -522,9 +484,41 @@ class PyKCS11Lib(object):
                     "nb_users": 0
                     }
 
+        # remember the lib file name
+        self.pkcs11dll_filename = pkcs11dll_filename
+
         # increase user number
         PyKCS11Lib._loaded_libs[pkcs11dll_filename]["nb_users"] += 1
 
+        return self
+
+    def unload(self):
+        """
+        unload the current instance of a PKCS#11 library
+        """
+
+        # in case NO library was found and used
+        if not hasattr(self, "pkcs11dll_filename"):
+            return
+
+        if self.pkcs11dll_filename not in PyKCS11Lib._loaded_libs:
+            raise PyKCS11Error(PyKCS11.LowLevel.CKR_GENERAL_ERROR,
+                               "invalid PyKCS11Lib state")
+
+        # decrease user number
+        PyKCS11Lib._loaded_libs[self.pkcs11dll_filename]["nb_users"] -= 1
+
+        if PyKCS11Lib._loaded_libs[self.pkcs11dll_filename]["nb_users"] == 0:
+            # unload only if no more used
+            self.lib.Unload()
+
+        # remove unused entry
+        # the case < 0 happens if lib loading failed
+        if PyKCS11Lib._loaded_libs[self.pkcs11dll_filename]["nb_users"] <= 0:
+            del PyKCS11Lib._loaded_libs[self.pkcs11dll_filename]
+
+        delattr(self, "pkcs11dll_filename")
+
     def initToken(self, slot, pin, label):
         """
         C_InitToken
@@ -812,6 +806,29 @@ class AES_GCM_Mechanism(object):
         return self._mech
 
 
+class AES_CTR_Mechanism(object):
+    """CKM_AES_CTR encryption mechanism"""
+
+    def __init__(self, counterBits, counterBlock):
+        """
+        :param counterBits: the number of incremented bits in the counter block
+        :param counterBlock: a 16-byte initial value of the counter block
+        """
+        self._param = PyKCS11.LowLevel.CK_AES_CTR_PARAMS()
+
+        self._source_cb = ckbytelist(counterBlock)
+        self._param.ulCounterBits = counterBits
+        self._param.cb = self._source_cb
+
+        self._mech = PyKCS11.LowLevel.CK_MECHANISM()
+        self._mech.mechanism = CKM_AES_CTR
+        self._mech.pParameter = self._param
+        self._mech.ulParameterLen = PyKCS11.LowLevel.CK_AES_CTR_PARAMS_LENGTH
+
+    def to_native(self):
+        return self._mech
+
+
 class RSAOAEPMechanism(object):
     """RSA OAEP Wrapping mechanism"""
 
@@ -901,6 +918,75 @@ class ECDH1_DERIVE_Mechanism(object):
         return self._mech
 
 
+class CONCATENATE_BASE_AND_KEY_Mechanism(object):
+    """CKM_CONCATENATE_BASE_AND_KEY key derivation mechanism"""
+
+    def __init__(self, encKey):
+        """
+        :param encKey: a handle of encryption key
+        """
+        self._mech = PyKCS11.LowLevel.CK_MECHANISM()
+        self._mech.mechanism = CKM_CONCATENATE_BASE_AND_KEY
+        self._mech.pParameter = encKey
+        self._mech.ulParameterLen = PyKCS11.LowLevel.CK_OBJECT_HANDLE_LENGTH
+
+    def to_native(self):
+        return self._mech
+
+
+class KEY_DERIVATION_STRING_DATA_MechanismBase(object):
+    """Base class for mechanisms using derivation string data"""
+
+    def __init__(self, data, mechType):
+        """
+        :param data: a byte array to concatenate the key with
+        :param mechType: mechanism type
+        """
+        self._param = PyKCS11.LowLevel.CK_KEY_DERIVATION_STRING_DATA()
+
+        self._data = ckbytelist(data)
+        self._param.pData = self._data
+        self._param.ulLen = len(self._data)
+
+        self._mech = PyKCS11.LowLevel.CK_MECHANISM()
+        self._mech.mechanism = mechType
+        self._mech.pParameter = self._param
+        self._mech.ulParameterLen = PyKCS11.LowLevel.CK_KEY_DERIVATION_STRING_DATA_LENGTH
+
+    def to_native(self):
+        return self._mech
+
+
+class CONCATENATE_BASE_AND_DATA_Mechanism(KEY_DERIVATION_STRING_DATA_MechanismBase):
+    """CKM_CONCATENATE_BASE_AND_DATA key derivation mechanism"""
+
+    def __init__(self, data):
+        """
+        :param data: a byte array to concatenate the key with
+        """
+        super().__init__(data, CKM_CONCATENATE_BASE_AND_DATA)
+
+
+class CONCATENATE_DATA_AND_BASE_Mechanism(KEY_DERIVATION_STRING_DATA_MechanismBase):
+    """CKM_CONCATENATE_DATA_AND_BASE key derivation mechanism"""
+
+    def __init__(self, data):
+        """
+        :param data: a byte array to concatenate the key with
+        """
+        super().__init__(data, CKM_CONCATENATE_DATA_AND_BASE)
+
+
+class XOR_BASE_AND_DATA_Mechanism(KEY_DERIVATION_STRING_DATA_MechanismBase):
+    """CKM_XOR_BASE_AND_DATA key derivation mechanism"""
+
+    def __init__(self, data):
+        """
+        :param data: a byte array to xor the key with
+        """
+        super().__init__(data, CKM_XOR_BASE_AND_DATA)
+
+
 class DigestSession(object):
     def __init__(self, lib, session, mecha):
         self._lib = lib
